!------------------------------------------------------------------
!
      program spell_12
!
!     Version 12.08 (i.e. aug 2012)
!
!------------------------------------------------------------------
!
! Given two 2D SpheroEllipses (or DiscoEllipses) S1, S2 centered 
! at (0,0) and (0,rcy(2)) and having orientations th1, th2,
! compute the contact distance by varying rcy(2) until the target
! function sab = 0.
!
! At each value of rcy(2), find the minimum of sigma(P2,S1), where P2
! is a point on S2, and sigma is the constraint potential of S1. This
! minimum value is called sab; it is our target function.
!
! At given rcy(2), the minimum sab is found by a Golden Search 
! [In 3D the Golden Search will be replaced by a Simplex Algorithm]
!
! The zero of sab(rcy(2)) is found by the Secant method (subroutine distsecant)
!
! Notation:
! - the centers have coordinates rcx and rcy;
! - the gliding points are at (xx,yy); within SHAKE they may shortly be 
!   at px,py, or ppx,ppy;
!
!--------------------------------------------------------------------------
!
      implicit double precision (a-h,o-z)
      logical test
      integer dta(8),dtb(8)
      character(10) ha(3),hb(3)
      common/rdin/nth,nt1,nt2
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      common/indices/ith1,ith2
      character(70) str1,str2    
!
      pi=4.d0*datan(1.0d0); pih=0.5d0*pi;
      tsc=1000.; tmn=60.*tsc; thr=60*tmn; tdy=24.*thr; ! for time calc
!
      open(unit=5,file="spell2d.par",form="formatted")
      open(unit=7,file="spell2d.out",form="formatted")
      open(unit=8,file="spell2d.rco",form="formatted") ! contact distances, table
!
      call readin;
!
      write(*,106) 
      write(7,106) 
106   format(1x/
     1  1x,"Output: file spell2d.rco contains, for each orientational",
     1  " config th1,th2, the contact distance rco."/
     1  1x,"Diagnostic values of rgl and utest (both should be",
     1  " small) are also written there."//
     1  )
!
! Parameters
! a) Physical 
      asq=aa*aa; csq=cc*cc; z1sq=z1*z1;
      x1=dsqrt(asq*(1.d0-z1sq/csq));
      z0=z1*(1.d0-asq/csq); z0sq=z0*z0;
      rcsq=asq*(1.d0-(z1sq/csq)*(1.d0-asq/csq));
      rc=sqrt(rcsq); ! cap radius; z0+rc is half length of SE
! b) Computational
      nt1l=0; nt1u=nth;
      nt2l=0; nt2u=nth;
      dth=2.d0*pi/dfloat(nth);
      if(test) then
        nt1l=min(nth,nt1); nt1u=max(nt1l,nt1);
        nt2l=min(nth,nt2); nt2u=max(nt2l,nt2);
      endif
      ngld=500;    nsec=200;     ! max steps golden search / secant search 
!
!   Shape matrix of ellipsoid:
!   cm0: matrix C in molecular (symmetric) coordinates
!   cm1_2: matrix C of S1_2 after rotation
!   rm1,rm2: rotation matrices of S1_2 in lab system 
!
      cm0(1,1)=1.d0/asq; cm0(1,2)=0.d0;
      cm0(2,1)=0.d0;     cm0(2,2)=1.d0/csq;
!   Shape matrix of caps:
      crc(1,1)=1.d0/rcsq; crc(1,2)=0.d0;
      crc(2,1)=0.d0;      crc(2,2)=1.d0/rcsq;
!   Correction factor at truncation:
      diva2=4.d0*(x1*x1/asq/asq+z1sq/csq/csq);
      diva=dsqrt(diva2);
      divc2=4.d0*(x1*x1+(z1-z0)*(z1-z0))/rcsq/rcsq;
      divc=dsqrt(divc2);
      gsig=diva/divc;
!-----------------------------------------------------------------------------
! The first spheroellipse is centered at (0,0), the second is placed on the z axis.
! theta1 and theta2 are scanned (outermost loop) to find the contact distances
! rco(th1,th2). For symmetry reasons only the range (0,pi/2) is relevant.
! If the boolean test=true, the theta double loop is restricted to one config
!-----------------------------------------------------------------------------
!
      rcx(1)=0.d0; rcy(1)=0.d0;      ! center of sell 1
      rcx(2)=0.d0;                   ! center (x co.) of sell 2
!
      secx=0.; sect=0.;  ! statistics: how often is nsec/tol exceeded? 
      gldx=0.; gldt=0.;  ! statistics: how often is ngld/tol exceeded? 
      ncgld=0; ncsig=0;  ! statistics: no of calls to SRs golden, sigma
! 
!      call date_and_time(ha(1),ha(2),ha(3),dta) ! start clock
!      time_begin=thr*dta(5)+tmn*dta(6)+tsc*dta(7)+dta(8);
      call clock(time_begin);
      do ith1=nt1l,nt1u ! theta1 loop
        th1=-pi+(ith1)*dth;
        cp1=dcos(th1); sp1=dsin(th1);
        call rotate(1,th1); ! compute rotated ellipse matrix cm1
        do ith2=nt2l,nt2u ! theta2 loop
          th2=-pi+(ith2)*dth;
          cp2=cos(th2); sp2=sin(th2);
          call rotate(2,th2); ! compute rotated ellipse matrix cm2
          call container; ! Compute a simple estimate for the initial distance rc20
          rcy(2)=rc20;
          call scout; ! starting guess for F and G
!
!--------------------------------------
! Distance loop: it all happens here
!
          if(test) write(*,201) th1,th2
201       format(1x,'th1,th2=',2f12.4)
          call distsecant(isfail);  
          if(isfail.eq.1) stop 43
!--------------------------------------
!
!    Hopefully, we have a contact distance
!    Check if rgl is small:
          call golden(1,2,igfail); if(igfail.eq.1) stop 544
          dx12=xx(2)-xx(1); dy12=yy(2)-yy(1);
          rgl=dsqrt(dx12*dx12+dy12*dy12); ! rgl should be small!
!    Check if normals at the gliders' positions are antiparallel (as they should be)
          call normal(1,xx(1),yy(1)); ! normal and tangent vectors in F (on ell1)
          call normal(2,xx(2),yy(2)); ! normal and tangent vectors in G (on ell2)
          un12=unx(1)*unx(2)+uny(1)*uny(2); 
          utest=1.d0-un12*un12; ! =0 if normals coincide
!    Write result:          
          dx12=rcx(2)-rcx(1); dy12=rcy(2)-rcy(1);
          rco=dsqrt(dx12*dx12+dy12*dy12);
          write(8,401) ith1,th1/pi,ith2,th2/pi,rco,rgl,utest
401       format(1x," it1,th1/pi,it2,th2/pi,rco,rgl,ut:",i5,f10.5,i5
     1     ,f10.5,3e13.5)
!
          if(ith1.eq.nt1.and.ith2.eq.nt2) then
            call writegp ! write a gnuplot script to draw this config
            print *,'writing gnuplot script for i,j=',ith1,ith2;
          endif
        enddo ! end theta2 loop 
        write(*,111) ith1,nth
111     format(1x,"ith1/nth",i5,"/",i5)
      enddo ! end theta1 loop
!
!      call date_and_time(hb(1),hb(2),hb(3),dtb) ! stop clock
!      time_end=thr*dtb(5)+tmn*dtb(6)+tsc*dtb(7)+dtb(8);
      call clock(time_end);
      write(*,112) (time_end-time_begin)/1000.;
      write(7,112) (time_end-time_begin)/1000.;
112   format(/1x,' Running time = ',f15.5,' sec')
      aden=(nth+1)*(nth+1);
      write(*,115) int(float(ncsig)/aden)
115   format(/1x,' no. calls to sigma per config. = ',i8)
      write(*,116) nt1,nt2
116   format(/1x,' wrote gnuplot script "spell2d.gnu" ',
     1 ' for config. ',i4,'/',i4)
!
      isecrat=int(100000*secx/sect)/1000;
      if(isecrat.ne.0) then
        str1=' pc of secant routines failed; try larger nsec';
        write(*,402) isecrat,str1
402     format(1x,i5,a)
      endif
      igldrat=int(100000*gldx/gldt)/1000;
      if(igldrat.ne.0) then
        str2=' pc of golden searches failed; try larger ngld';
        write(*,402) igldrat,str2
      endif
!
      stop
      end  
!----------------------------------------------------
!
      subroutine readin
!
! Read parameters  
!
      implicit double precision (a-h,o-z)
      logical test
      character(70) headline,ident,sdummy    
      common/rdin/nth,nt1,nt2
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
!
      pi=4.d0*datan(1.0d0); pih=0.5d0*pi;
!
      read(5,'(a/a)') sdummy,ident
      write(*,101) ident
      write(7,101) ident
101   format(1x/
     1  "---------------------------------------------------------"//
     1  "     Contact distance between two SpheroEllipses        " /
     1  "         Program: spell_12 / version 12.08              " /
     1  "         Author: F. J. Vesely                           " //
     1  "---------------------------------------------------------"/
     1  /1x,a)
      read(5,'(a/3f10.5)') sdummy,aa,cc,z1
      z1=dabs(z1);
      write(*,102) aa,cc,z1
      write(7,102) aa,cc,z1
102   format(1x/
     1  1x,"Particle shape: ", 
     1  1x,"  aa  = ",f8.2,",      bb  =",f8.2
     1  ,",        z1   =",f8.2 
     1  )
      read(5,'(a/3i10,l5/a/e10.1)') sdummy,nth,nt1,nt2,test,dummy,tol
      nt1=min(nt1,nth); nt2=min(nt2,nth);
      write(*,105) nth,nt1,nt2,test,tol
      write(7,105) nth,nt1,nt2,test,tol
105   format(1x,"Computational :",
     1  1x,"  nth   =",i8,",     nt1  =",i8,",       nt2   =",i8
     1  ,",    test =",l4 /1x,"tol= ",1Pe10.1
     1  )
!
      return
      end
!
!----------------------------------------------------
!
      subroutine rotate(ise,theta)
!
! Rotate base ellipse ise (=1 or 2) by theta
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      dimension rm(2,2),rn(2,2)
!
      elx=0.d0; ely=cc; elm=ely;
      th=theta; cp=dcos(th); sp=dsin(th);
      rm(1,1)=cp; rm(1,2)=-sp;
      rm(2,1)=sp; rm(2,2)=cp;  ! rotation matrix, 2D
      rn(1,1)=rm(1,1); rn(1,2)=rm(2,1); ! inverse rotation matrix
      rn(2,1)=rm(1,2); rn(2,2)=rm(2,2);
! Rotate ellipse:
      cmt(1,1)=cm0(1,1)*rm(1,1)+cm0(1,2)*rm(2,1);
      cmt(1,2)=cm0(1,1)*rm(1,2)+cm0(1,2)*rm(2,2);
      cmt(2,1)=cm0(2,1)*rm(1,1)+cm0(2,2)*rm(2,1);
      cmt(2,2)=cm0(2,1)*rm(1,2)+cm0(2,2)*rm(2,2);
      if(ise.ne.1.and.ise.ne.2) stop "ise invalid"
      if(ise.eq.1) then
        cm1(1,1)=rm(1,1)*cmt(1,1)+rm(2,1)*cmt(2,1);
        cm1(1,2)=rm(1,1)*cmt(1,2)+rm(2,1)*cmt(2,2);
        cm1(2,1)=rm(1,2)*cmt(1,1)+rm(2,2)*cmt(2,1);
        cm1(2,2)=rm(1,2)*cmt(1,2)+rm(2,2)*cmt(2,2);
      else 
        cm2(1,1)=rm(1,1)*cmt(1,1)+rm(2,1)*cmt(2,1);
        cm2(1,2)=rm(1,1)*cmt(1,2)+rm(2,1)*cmt(2,2);
        cm2(2,1)=rm(1,2)*cmt(1,1)+rm(2,2)*cmt(2,1);
        cm2(2,2)=rm(1,2)*cmt(1,2)+rm(2,2)*cmt(2,2);
      endif
      elxx=rn(1,1)*elx+rn(1,2)*ely;
      elyy=rn(2,1)*elx+rn(2,2)*ely;
      ulx(ise)=elxx/elm; uly(ise)=elyy/elm;
!
      return
      end  
!
!----------------------------------------------------
      subroutine container
!
! A simple estimate for the initial distance: enclose the particles in rectangles
! whose upper/lowermost corner points should be just clear of each other 
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
!
      usy=-ulx(1); ! y-component of short axis unit vct (clockwise from long ax.)  
      py1=(z0+rc)*uly(1)+aa*usy; ! one corner of container box 1  
      py2=(z0+rc)*uly(1)-aa*usy; ! one corner of container box 1  
      py3=-(z0+rc)*uly(1)-aa*usy; ! one corner of container box 1  
      py4=-(z0+rc)*uly(1)+aa*usy; ! one corner of container box 1
      pym=max(py1,py2); pym=max(pym,py3); pym=max(pym,py4); ! highest pt of container 1    
      usy=-ulx(2); ! y-component of short axis unit vct (clockwise from long ax.)  
      qy1=(z0+rc)*uly(2)+aa*usy; ! one corner of container box 2   
      qy2=(z0+rc)*uly(2)-aa*usy; ! one corner of container box 2  
      qy3=-(z0+rc)*uly(2)-aa*usy; ! one corner of container box 2  
      qy4=-(z0+rc)*uly(2)+aa*usy; ! one corner of container box 2
      qym=min(qy1,qy2); qym=min(qym,qy3); qym=min(qym,qy4); ! lowest pt of container 2 
      rc20=pym+abs(qym)+0.1d0*aa;
!
      return
      end
!
!---------------------------------------------------
      subroutine scout
!
! Choose starting points on the SpheroEllipses: intersections
! of center-center line with S1 and S2:
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      dimension rm(2,2)
!
! Solve lam**2 * (r12*C1*r12)-1=0 for lam; then f0=r1+lam*r12
! is the intersection pt, with r12 the center-center vector
!
       do i=1,2
        j=3-i;
        rc12x=rcx(j)-rcx(i);     ! always zero in our lab system! ***
        rc12y=rcy(j)-rcy(i);     ! center-center vector
        rc12sq=rc12x*rc12x+rc12y*rc12y;  
        rcsq=rc*rc; 
        gg=rc12x*ulx(i)+rc12y*uly(i);
        sg=dsign(1.d0,gg);
!
        if(i.eq.1) then
          crx=cm1(1,1)*rc12x+cm1(1,2)*rc12y; 
          cry=cm1(2,1)*rc12x+cm1(2,2)*rc12y;      
        else
          crx=cm2(1,1)*rc12x+cm2(1,2)*rc12y; 
          cry=cm2(2,1)*rc12x+cm2(2,2)*rc12y;      
        endif
        rcr=rc12x*crx+rc12y*cry;
        xlam=dsqrt(1.d0/rcr);
        ffx=rcx(i)+xlam*rc12x;      
        ffy=rcy(i)+xlam*rc12y;  
        prj=xlam*gg;
        if(prj*prj.gt.z1sq) then ! treat caps
          hh=sg*z0*gg/rc12sq;
          rt=dsqrt(hh*hh-z0*z0/rc12sq+rcsq/rc12sq);
          xlam=hh+rt;
          ffx=rcx(i)+xlam*rc12x;      
          ffy=rcy(i)+xlam*rc12y;  
        endif
        xx(i)=ffx; yy(i)=ffy;   ! pos. of glider at start of SR golden
      enddo
!
      return
      end
!
!----------------------------------------------------
!
      subroutine distsecant(isfail)
!
! Vary position rcy(2) until Sigma(rc_12)=0  ==> contact!
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      logical bra,rtarget;
      pi=4.d0*datan(1.d0); pih=0.5d0*pi;
! 
      isfail=0;
      isec=0; y20=rcy(2);
! Estimate 1:  
      isec=isec+1; 
      y21=y20; rcy(2)=y21; ! trivial 
      call dstep;
      sab1=sgm(2,1);  
      if(sab1.le.0.d0) then
        write (*,202) sab1;
202     format(1x,'sab1=',e14.6)
        stop " Initial distance too small!"  
      endif
! end estimate 1
!
! Estimate 2:
      isec=isec+1; 
      zstep=-0.05d0*aa; 
      y22=y21+zstep; rcy(2)=y22; yy(2)=yy(2)+zstep;
      call dstep;
      sab2=sgm(2,1);  ! pt 2 in E1
! end estimate 2
!
! Hopefully, we have an interval [y21,y22] that is close to the root.
! Now do Secant method:
!
      fl=sab1; f=sab2;
      if(dabs(fl).lt.dabs(f)) then
        root=y21; yl=y22;
        swap=fl; fl=sab2; f=swap;
      else
        root=y22; yl=y21;
      endif 
!
      rtarget=.false.;  
      do while (.not.rtarget) 
        isec=isec+1;
        dy=(yl-root)*f/(f-fl);
        yl=root;
        fl=f;
        root=root+dy; rcy(2)=root;
        call dstep;
        f=sgm(2,1);
        if(dabs(dy).lt.tol.or.f.eq.0.) then
          rtarget=.true.;
        endif
        if(isec.gt.nsec) then
          secx=secx+1; rtarget=.true.; 
          isfail=1;
        endif
      enddo
      sect=sect+1.;
!
      return
      end 

!----------------------------------------------------
      subroutine dstep
!
! distance step within the Secant subroutine
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
!
      call scout;              ! starting positions of gliding pts on their contours
      call golden(2,1,igfail); ! find minimum of sigma(ipt=2,ise=1):
                               ! pt 2 glides on S2, finds min of "field" S1
      ncgld=ncgld+1;
      if(igfail.eq.1) then
        write(*,301)
        write(7,301)
301     format(1x,'dstep:igfail=1')
        stop 54
      endif
!
      return
      end

!--------------------------------------------------------------
!
      subroutine golden(ipt,ise,igfail)
!
! Given the spheroellipsoids S(ipt) and S(ise), let a point (xx,yy) glide 
! along the contour of S(ipt) until sigma(xx,yy,ise) assumes a minimum
! Method: Golden Section
!
      implicit double precision (a-h,o-z)
      logical test,lbra,ldig
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      common/indices/ith1,ith2
!
! To prepare for the Golden Search we first have to find a triplet
! of abscissae x0-x2 and function values qq0-qq2:
!
      pi=4.d0*datan(1.d0); pih=0.5d0*pi;
      tsc=1000.; tmn=60.*tsc; thr=60.*tmn; tdy=24.*thr; ! for timer calc
      igfail=0;
      qq0=0.d0; qq1=0.d0; qq2=0.d0; qq3=0.d0; 
      x0=0.d0; x1=0.d0; x2=0.d0; x3=0.d0;
!
! Tell me starting value Q0=sigma(rip(ipt),ise):
      rx0=xx(ipt); ry0=yy(ipt)
      x0=0.d0;
      call normal(ipt,rx0,ry0); ! output un,ut on contour ipt
      call normal(ise,rx0,ry0); ! output un,ut on the equipotential line of SE(ise)
      uttx=utx(ipt); utty=uty(ipt);
      call sigma(rx0,ry0,ise,qq0); ! Starting value 
!
! Try to find a starting triplet:
! Now, if the normal to the glider contour S(ipt) and the gradient of the
! field sigma(ise) are parallel, we are already at an extremum of sigma:
      check=unx(ipt)*unx(ise)+uny(ipt)*uny(ise);
      check=check*check-1.d0;
      if((check*check).lt.1.d-16) then
        sgm(ipt,ise)=qq0; 
        gldt=gldt+1; 
        return ! leaving SR golden successfully
      endif
!
! If we are not at the minimum, then we can find the direction TOWARDS it:
! it is the projection of the negative gradient of sigma(ise) onto the tangent of SE(ipt):
      st=-dsign(1.d0,(uttx*unx(ise)+utty*uny(ise)));
! Trial step:
      x1=0.05d0*st*aa; 
      px=rx0+x1*uttx; py=ry0+x1*utty
      call shake(ipt,px,py,rx1,ry1,sg1); ! bring it back onto S(ipt)
      call sigma(rx1,ry1,ise,qq1); ! compute how deep it is in the ise potential
!
! Now we have two starting values x0,x1 with qq1>=qq0;
! First guess for x2:
      x2=2.d0*x1;
      px=rx0+x2*uttx; py=ry0+x2*utty;
      call shake(ipt,px,py,rx2,ry2,sg2);
      call sigma(rx2,ry2,ise,qq2);
!
! If arrived, we have a starting triplet x0,x1,x2; otherwise, go on:
!
      lbra=.false.;
      if(qq1.lt.qq2) lbra=.true.;
      ipre=0;
      do while(.not.lbra)
        ipre=ipre+1;
        rx0=rx1; ry0=ry1; qq0=qq1;
        call normal(ipt,rx0,ry0);
        uttx=utx(ipt); utty=uty(ipt);
        x1=x2-x1; 
        px=rx0+x1*uttx; py=ry0+x1*utty;
        call shake(ipt,px,py,rx1,ry1,sg1);
        call sigma(rx1,ry1,ise,qq1);
        x2=2.d0*x1;
        px=rx0+x2*uttx; py=ry0+x2*utty;
        call shake(ipt,px,py,rx2,ry2,sg2);
        call sigma(rx2,ry2,ise,qq2);
        if(qq1.lt.qq0.and.qq1.lt.qq2) lbra=.true.;
        if(ipre.gt.ngld) then
          lbra=.true.;
          igfail=1;
        endif
      enddo
      if(igfail.ne.0) then
        write(*,401) igfail;
        write(7,401) igfail;
401     format(1x,'igfail=',i3)
        write(*,402) ith1,ith2,ipre,qq0,qq1,qq2
        write(7,402) ith1,ith2,ipre,qq0,qq1,qq2
402     format(1x,'gld-6: ith1-2,ipre,qq0-2=',3i5,3e14.6)
        stop 55
      endif
!
! By now we should definitely have a starting triplet x0,x1,x2;
! Golden Search starts now:
!
! Displace the inner point (x1 -> rr1):
      x1=x0+(x2-x0)*0.38197d0; 
      rxo=rx0; ryo=ry0;
      px=rx0+x1*uttx; py=ry0+x1*utty;
      call shake(ipt,px,py,rx1,ry1,sg1);
      call sigma(rx1,ry1,ise,qq1);
! Now we go:
      cg=2.d0/(3.d0+sqrt(5.d0)); rg=1.d0-cg;
      x3=x2; qq3=qq2;
      if(abs(x2-x1).gt.abs(x1-x0)) then
        x1=x1; x2=x1+cg*(x1-x0);
      else
        x2=x1; x1=x1-cg*(x1-x0); 
      endif
      px=rxo+x1*uttx; py=ryo+x1*utty;
      call shake(ipt,px,py,rx1,ry1,sg1);
      call sigma(rx1,ry1,ise,qq1);
      px=rxo+x2*uttx; py=ryo+x2*utty;
      call shake(ipt,px,py,rx2,ry2,sg2); 
      call sigma(rx2,ry2,ise,qq2);
!     
      ldig=.false.;   ! gold dig successful?
      igld=0;
      do while(.not.ldig)
        igld=igld+1;
        if(qq2<qq1) then
          x0=x1; x1=x2; qq0=qq1; qq1=qq2; 
          rx0=rx1; ry0=ry1;
          rx1=rx2; ry1=ry2;
          x2=rg*x1+cg*x3;
          px=rxo+x2*uttx; py=ryo+x2*utty;
          call shake(ipt,px,py,rx2,ry2,sg2);
          call sigma(rx2,ry2,ise,qq2);
        else
          x3=x2; x2=x1; qq3=qq2; qq2=qq1;
          rx3=rx2; ry3=ry2;
          rx2=rx1; ry2=ry1;
          x1=rg*x2+cg*x0;
          px=rxo+x1*uttx; py=ryo+x1*utty;
          call shake(ipt,px,py,rx1,ry1,sg1);
          call sigma(rx1,ry1,ise,qq1);
        endif
        a3m0=dabs(x3-x0); 
        ta1p2=tol*(dabs(x1)+dabs(x2));
        if(a3m0.le.ta1p2) ldig=.true.;
        if(igld.gt.ngld) then
          igfail=1; gldx=gldx+1.; ldig=.true.; 
        endif
      enddo
!
      if(qq1<qq2) then
        qq8=qq1; xmin=x1; rdx=rx1; rdy=ry1;
      else
        qq8=qq2; xmin=x2; rdx=rx2; rdy=ry2;
      endif
      xx(ipt)=rdx; yy(ipt)=rdy;
      call sigma(rdx,rdy,ise,sig);
      sgm(ipt,ise)=sig; 
!
      gldt=gldt+1.;
      if(test) print *,'leaving golden,igfail,sig=',igfail,sig
!
      return
      end
!
!--------------------------------------------------------------
!
      subroutine shake(ise,px,py,ppx,ppy,sigpp)
!
!  Given a spheroellipse S(ise) and a point P(px,py) near its contour, 
!  produce P'(ppx,ppy) such that it is exactly on the contour of S(ise).
!  Input: ise, px,py, and gradxy at the nearby base point (xx,yy) on the
!  contour of S(ise).
!  For details, look up the SHAKE algorithm of Ryckaert et al. 
!
      implicit double precision (a-h,o-z)
      logical test,succ
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      dimension cmc(2,2);  ! current SE matrix
!
      succ=.false.; 
!
      if(ise.eq.1) then
        cmc(1,1)=cm1(1,1); cmc(1,2)=cm1(1,2);
        cmc(2,1)=cm1(2,1); cmc(2,2)=cm1(2,2); 
      else
        cmc(1,1)=cm2(1,1); cmc(1,2)=cm2(1,2);
        cmc(2,1)=cm2(2,1); cmc(2,2)=cm2(2,2); 
      endif
      rqx=px-rcx(ise); rqy=py-rcy(ise);
      tmpx=cmc(1,1)*rqx+cmc(1,2)*rqy;
      tmpy=cmc(2,1)*rqx+cmc(2,2)*rqy;   ! grad(sigma) in P, if near ellipse body
      prj=rqx*ulx(ise)+rqy*uly(ise);          
      gs=1.d0;
      if (prj*prj.gt.z1sq) then         ! if P near circular cap
        z0x=rcx(ise)+z0*dsign(1.d0,prj)*ulx(ise);
        z0y=rcy(ise)+z0*dsign(1.d0,prj)*uly(ise);
        rqx=px-z0x; rqy=py-z0y;
        tmpx=crc(1,1)*rqx+crc(1,2)*rqy;
        tmpy=crc(2,1)*rqx+crc(2,2)*rqy; 
        gs=gsig; 
      endif
      grdpx=2.d0*gs*tmpx; grdpy=2.d0*gs*tmpy; ! gradient in P
!
! With the new position pxy the constraint sigma is poorly fulfilled:
      call sigma(px,py,ise,sigp); ! sigp not equal 0
! so: create a second estimate:
      ag=sigp/(gradx(ise)*grdpx+grady(ise)*grdpy);  ! = gamma*a
      ppx=px-ag*gradx(ise);
      ppy=py-ag*grady(ise);   ! P'... improved position
! check if P' is on SE:
      call sigma(ppx,ppy,ise,sigpp);
!
      ish=2;
      do while (ish.le.20.and.dabs(sigpp).gt.tol)
! ... and a third,... one:
        px=ppx; py=ppy;
        rqx=px-rcx(ise); rqy=py-rcy(ise);
        sigp=sigpp;
        tmpx=cmc(1,1)*rqx+cmc(1,2)*rqy;
        tmpy=cmc(2,1)*rqx+cmc(2,2)*rqy;   ! grad(sigma) in P, if near ellipse body
        prj=rqx*ulx(ise)+rqy*uly(ise);     
        if (prj*prj.gt.z1sq) then         ! if P is near circular cap
          z0x=rcx(ise)+z0*dsign(1.d0,prj)*ulx(ise);
          z0y=rcy(ise)+z0*dsign(1.d0,prj)*uly(ise);
          rqx=px-z0x; rqy=py-z0y;
          tmpx=gsig*(crc(1,1)*rqx+crc(1,2)*rqy);
          tmpy=gsig*(crc(2,1)*rqx+crc(2,2)*rqy);  
        endif
        grdpx=2.d0*tmpx; grdpy=2.d0*tmpy; 
        ag=sigp/(gradx(ise)*grdpx+grady(ise)*grdpy);  ! = gamma*a
        ppx=px-ag*gradx(ise);
        ppy=py-ag*grady(ise);              ! P' ...improved position 
        call sigma(ppx,ppy,ise,sigpp);
        ish=ish+1
      enddo
!
      return
      end
!---------------------------------------------------
!
      subroutine normal(ise,qx,qy)
!
! Given a SpheroEllipse S(ise) and a point Q(qx,qy),
! find the gradient of the field sigma(Q,ise)
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      dimension cmc(2,2);  ! current SE matrix
!
      if(ise.eq.1) then   ! which ellipse matrix?
        cmc(1,1)=cm1(1,1); cmc(1,2)=cm1(1,2);
        cmc(2,1)=cm1(2,1); cmc(2,2)=cm1(2,2); 
      else
        cmc(1,1)=cm2(1,1); cmc(1,2)=cm2(1,2);
        cmc(2,1)=cm2(2,1); cmc(2,2)=cm2(2,2); 
      endif
      rqx=qx-rcx(ise); rqy=qy-rcy(ise);
      unxx=2.d0*(cmc(1,1)*rqx+cmc(1,2)*rqy);
      unyy=2.d0*(cmc(2,1)*rqx+cmc(2,2)*rqy);  
      gradx(ise)=unxx; grady(ise)=unyy;  ! ellipse body
      prj=rqx*ulx(ise)+rqy*uly(ise);     ! proj. of (rip-rcc) on long axis       
      if (prj*prj.gt.z1sq) then          !circular cap
        z0x=rcx(ise)+z0*dsign(1.d0,prj)*ulx(ise);
        z0y=rcy(ise)+z0*dsign(1.d0,prj)*uly(ise);
        rqx=qx-z0x; rqy=qy-z0y;
        unxx=2.d0*gsig*(crc(1,1)*rqx+crc(1,2)*rqy);
        unyy=2.d0*gsig*(crc(2,1)*rqx+crc(2,2)*rqy);  
        gradx(ise)=unxx; grady(ise)=unyy;
      endif
      unm=dsqrt(unxx*unxx+unyy*unyy);
      unx(ise)=unxx/unm; uny(ise)=unyy/unm;   ! unit gradient vector in Q 
      utx(ise)=-uny(ise); uty(ise)= unx(ise); ! unit tangent vector in Q, 
                                              ! counter-lockwise of gradient
!
      return
      end  
!
!----------------------------------------------------
      subroutine sigma(qx,qy,ise,sig)
!
! Given a point Q(qx,qy) and a SpheroEllipse S(ise) with center at R, 
! calculate the value of the"constraint potential" sigma=(Q-R)*C(ise)*(Q-R)-1
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      dimension cmc(2,2);  ! current SE (body) matrix
!
      ncsig=ncsig+1;
      if(ise.eq.1) then ! which ellipse matrix?
        cmc(1,1)=cm1(1,1); cmc(1,2)=cm1(1,2);
        cmc(2,1)=cm1(2,1); cmc(2,2)=cm1(2,2); 
      else
        cmc(1,1)=cm2(1,1); cmc(1,2)=cm2(1,2);
        cmc(2,1)=cm2(2,1); cmc(2,2)=cm2(2,2); 
      endif
      rqx=qx-rcx(ise); rqy=qy-rcy(ise);
      tmpx=cmc(1,1)*rqx+cmc(1,2)*rqy;
      tmpy=cmc(2,1)*rqx+cmc(2,2)*rqy;   
      sig=rqx*tmpx+rqy*tmpy-1.d0; ! ellipse body
      prj=rqx*ulx(ise)+rqy*uly(ise);
      if(prj*prj.gt.z1sq) then    ! circular cap
        z0x=rcx(ise)+z0*dsign(1.d0,prj)*ulx(ise);
        z0y=rcy(ise)+z0*dsign(1.d0,prj)*uly(ise);
        rqx=qx-z0x; rqy=qy-z0y;
        tmpx=crc(1,1)*rqx+crc(1,2)*rqy;
        tmpy=crc(2,1)*rqx+crc(2,2)*rqy;
        sig=gsig*(rqx*tmpx+rqy*tmpy-1.d0);  
      endif
!
      return
      end
!
!----------------------------------------------------
!
      subroutine writegp
!
! Print gnuplot scripts:
! spell2d.gnu draws the last config of the double loop;
! sp12.gnu draws first the contact distance rco(th1,th2),
! then, as a check, rgl(th1,th2); the latter should be small 
! for all orientations - it is the residual glider distance 
! at contact
!
      implicit double precision (a-h,o-z)
      logical test
      common aa,cc,z1,z1sq,th1,th2,dth,z0,rc,rcsq,gsig,cp1,sp1,cp2,sp2
     1 ,sigp,rc20,utest,tol,secx,sect,gldx,gldt
     1 ,isec,nsec,ngld,ncgld,ncsig,test
      common/matrices/cm0(2,2),cm1(2,2),cm2(2,2),crc(2,2),cmt(2,2),
     1  rm1(2,2),rm2(2,2),sgm(2,2)
      common/vectors/rcx(2),rcy(2),unx(2),uny(2),utx(2),uty(2),
     1  ulx(2),uly(2),xx(2),yy(2),gradx(2),grady(2),rc12x,rc12y
      dimension rm(2,2)
!
      open(unit=4,file="spell2d.gnu",form="formatted")  ! for gnuplot display
      rewind 4
!
      call normal(1,xx(1),yy(1)); ! normal and tangent vectors in F (on ell1)
      call normal(2,xx(2),yy(2)); ! normal and tangent vectors in G (on ell2)
      write(4,600) aa,cc,z1,xx(1),yy(1),xx(2),yy(2),gradx(2),grady(2)
     1 ,utx(1),uty(1)
600   format(2x,'set parametric; set terminal wxt; set sample 1000'
     1 /2x,'ph=pi/2.;'
     1 /2x,'a=',f10.5,'; c=',f10.5,'; z1=',f10.5,';'
     1 /2x,'ffx=',e12.4,'; ffy=',e12.4
     1 /2x,'ggx=',e12.4,'; ggy=',e12.4
     1 /2x,' utfx=',f10.5,'; utfy=',f10.5,';'
     1 /2x,' utgx=',f10.5,'; utgy=',f10.5,';'
     1 /2x,'tfax=ffx-0.5*utfx; tfay=ffy-0.5*utfy; '
     1 /2x,'tfbx=ffx+0.5*utfx; tfby=ffy+0.5*utfy;' 
     1 /2x,'tgax=ggx-0.5*utgx; tgay=ggy-0.5*utgy; '
     1 /2x,'tgbx=ggx+0.5*utgx; tgby=ggy+0.5*utgy;'
     1 )
      write(4,601) cp1,sp1,cp2,sp2 
601   format(2x,'cp1=',f10.5,'; sp1=',f10.5/1x,'cp2=',f10.5,
     1 '; sp2=',f10.5)
      write(4,602) rcx(1),rcy(1),rcx(2),rcy(2)
602   format(2x,'x1=',f10.5,'; y1=',f10.5/1x,'x2=',f10.5,'; y2='
     1 ,f10.5)
      write(4,619)
619   format(2x,
     1 'asq=a*a; csq=c*c; z1sq=z1*z1;  z0=z1*(1.-asq/csq)'
     1 /2x,'rcsq=asq*(1.-(z1sq/csq)*(1.-asq/csq)); rc=sqrt(rcsq);'
     1 /2x,' ; zbr=z0/rc; zbrsq=zbr*zbr;'
     1 /2x,' t1=atan(z1/sqrt(csq-z1sq+1.e-12)); st1=sin(t1);'
     1 )
      yl=yy(2)-1.5*(z1+rc); yu=yy(2)+1.5*(z1+rc);
      write(4,613) 
613   format(2x,'set xrange [-1.5*(z1+rc) : 1.5*(z1+rc)];'
     1 /2x,'set yrange [-1.5*(z1+rc) : 1.5*(z1+rc)];'
     1 /2x, ' set nokey;'
     1 )
      write(4,713)
713   format(2x,'set size ratio -1;'
     1 /2x,'set arrow from ffx,ffy to ggx,ggy nohead;'
     1 /2x,'set arrow from tfax,tfay to tfbx,tfby nohead;'
     1 /2x,'set arrow from tgax,tgay to tgbx,tgby nohead;'
     2 )
      write(4,803)
803   format('# elliptic body part:'
     1 /2x,'xe(t)=a*cos(t); ye(t)=c*sin(t);'
     1 /'# phi (polar angle) from t:'
     1 /2x,'gam(t)=sgn(cos(t)); sig(t)=sgn(sin(t));'
     1 /2x,'phi(t)=gam(t)*sig(t)*atan(a/c/tan(t));'
     1 /'# ... and beta derives from phi:'
     1 /2x,'sph(t)=sin(phi(t)); cph(t)=cos(phi(t));'
     1 /2x,'argb(t)=-zbr*sph(t)**2+cph(t)*sqrt(1.-zbrsq*sph(t)**2);'
     1 /2x,'beta(t)=acos(argb(t));'
     1 /2x,'xc(t)=gam(t)*rc*sin(beta(t));'
     1 /2x,'yc(t)=sig(t)*(z0+rc*cos(beta(t)));'
     1 /2x,'x(t)= abs(sin(t)) <= st1 ? xe(t) : xc(t);'
     1 /2x,'y(t)= abs(sin(t)) <= st1 ? ye(t) : yc(t);')
       write(4,603)
603   format(1x,'plot [0:2.*pi] x1+cp1*x(t)+sp1*y(t),',
     1 'y1-sp1*x(t)+cp1*y(t),',
     1 'x2+cp2*x(t)+sp2*y(t),y2-sp2*x(t)+cp2*y(t),',
     1 'ffx+0.1*cos(t),ffy+0.1*sin(t),',
     1 'ggx+0.1*cos(t),ggy+0.1*sin(t);')
      write(4,604)
604   format(1x,'pause 2;')
      close(4)
!
      open(unit=45,file="sp12.gnu",form="formatted") 
      write(45,605)
605   format(1x,
     1 'splot "./spell2d.rco" u 3:5:6;'/
     1 1x,'pause 2;'/
     1 'splot "./spell2d.rco" u 3:5:7;')
      write(45,604)
!
      return
      end
!

!---------------------------------------------------------
!
      subroutine clock(wct)
!
! Looks at wall clock
!
      implicit double precision (a-h,o-z)
      integer dct(8)
      character(10) hct(3)
!      
      tsc=1000.; tmn=60.*tsc; thr=60.*tmn; tdy=24.*thr; ! for timer calc
      call date_and_time(hct(1),hct(2),hct(3),dct) ! read clock
      wct=thr*dct(5)+tmn*dct(6)+tsc*dct(7)+dct(8)
!
      return
      end
!-----------------------------------
      
